from pettingzoo.utils.depreciated_module import DepreciatedModule

double_dunk_v0 = DepreciatedModule("double_dunk", "v0", "v1")
entombed_competitive_v0 = DepreciatedModule("entombed_competitive", "v0", "v1")
joust_v0 = DepreciatedModule("joust", "v0", "v1")
mario_bros_v0 = DepreciatedModule("mario_bros", "v0", "v1")
maze_craze_v0 = DepreciatedModule("maze_craze", "v0", "v1")
othello_v0 = DepreciatedModule("othello", "v0", "v1")
pong_quadrapong_v0 = DepreciatedModule("pong_quadrapong", "v0", "v1")
tennis_v0 = DepreciatedModule("tennis", "v0", "v1")
video_checkers_v0 = DepreciatedModule("video_checkers", "v0", "v1")
warlords_v0 = DepreciatedModule("warlords", "v0", "v1")
wizard_of_wor_v0 = DepreciatedModule("wizard_of_wor", "v0", "v1")
