from pettingzoo.utils.depreciated_module import DepreciatedModule

prison_v0 = DepreciatedModule("prison", "v0", "v1")
prospector_v0 = DepreciatedModule("prospector", "v0", "v1")
knights_archers_zombies_v0 = DepreciatedModule("knights_archers_zombies", "v0", "v1")
